use scraper::Html;

use super::{Extractor, trim_whitespace, parse_date};

#[derive(Debug, Default)]
pub struct Label {
    pub artist_name: String,
    pub artist_url: String,
    pub date: String, 
    pub author: String,
    pub author_url: String,
}

impl Extractor for Label {
    type Output = Self;
    fn extract_all(document: Html) -> Vec<Self> {
        let mut artists : Vec<Self> = Vec::new();

        let selector = scraper::Selector::parse("ul>li").unwrap();
        let select = document.select(&selector);
        for el in select {
            artists.push(Self::extract(Html::parse_fragment(&el.html())));
        }

        artists
    }

    fn extract(document: Html) -> Self {
      let a_el = document.select(&scraper::Selector::parse("a").unwrap()).next().unwrap();

      let title = a_el.inner_html();
      let url = a_el.value().attr("href").unwrap();

      let more_info_el = document.select(&scraper::Selector::parse("div").unwrap()).next().unwrap();
      let date = more_info_el.inner_html();

      let author_el = more_info_el.select(&scraper::Selector::parse("a").unwrap()).next().unwrap();
      let author = author_el.inner_html();
      let author_url = author_el.value().attr("href").unwrap();

      Self {
          artist_name: trim_whitespace(&title),
          artist_url: trim_whitespace(url),
          date: parse_date(&date),
          author: trim_whitespace(&author),
          author_url: trim_whitespace(author_url),
      }
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    const EXAMPLES: &'static str = 
r#"
<ul>
  <li>
    <a class="symbol__company" href="/labels/arcanumania-records/">ARCANUMANIA Records.</a>
    <div class="h5 any--no-wrap">
      2022-06-13 21:27:11 by 
      <a class="user a--inherit" data-icon="" data-is-vip="0" href="/users/haru/">haru</a>
    </div>
  </li>
  <li>                 
    <a class="symbol__company" href="/labels/omega-code/">Omega Code</a>
    <div class="h5 any--no-wrap">
      2022-06-12 18:43:41 by 
      <a class="user a--inherit" data-icon="" data-is-vip="1" href="/users/inartistic/">inartistic</a>
    </div>
  </li>
</ul>
"#;

  #[test]
  fn extract_all_must_return_two_result() {
    let document = scraper::Html::parse_fragment(EXAMPLES);
    let artists = Label::extract_all(document);
    assert_eq!(artists.len(), 2);
    assert_eq!(artists[0].artist_name, "ARCANUMANIA Records.");
    assert_eq!(artists[0].artist_url, "/labels/arcanumania-records/");
    assert_eq!(artists[0].date, "2022-06-13 21:27:11");
    assert_eq!(artists[0].author, "haru");
    assert_eq!(artists[0].author_url, "/users/haru/");
    assert_eq!(artists[1].artist_name, "Omega Code");
    assert_eq!(artists[1].artist_url, "/labels/omega-code/");
    assert_eq!(artists[1].date, "2022-06-12 18:43:41");
    assert_eq!(artists[1].author, "inartistic");
    assert_eq!(artists[1].author_url, "/users/inartistic/");
  }

  const EXAMPLE: &'static str = r#"
  <a class="symbol__company" href="/labels/arcanumania-records/">ARCANUMANIA Records.</a>
    <div class="h5 any--no-wrap">
      2022-06-13 21:27:11 by 
      <a class="user a--inherit" data-icon="" data-is-vip="0" href="/users/haru/">haru</a>
    </div>
  "#;

  #[test]
  fn extract_must_be_return_correct_value() {
    let document = scraper::Html::parse_fragment(EXAMPLE);
    let artist = Label::extract(document);
    assert_eq!(artist.author, "haru");
    assert_eq!(artist.author_url, "/users/haru/");
    assert_eq!(artist.artist_name, "ARCANUMANIA Records.");
    assert_eq!(artist.artist_url, "/labels/arcanumania-records/");
    assert_eq!(artist.date, "2022-06-13 21:27:11");
  }
}