use std::env;

use scraper::Html;

mod artists;
mod labels;
mod musicians;
mod releases;

pub trait Extractor {
    type Output;
    fn extract_all(select: Html) -> Vec<Self::Output>;
    fn extract(select: Html) -> Self::Output;
}

const RECENTLY_UPDATED_SELECTOR : &'static str= "body>div.c2>div";
const CATEGORY_TITLE_SELECTOR : &'static str= "h2>a";
const NEWS_TITLE_SELECTOR : &'static str= "div.text";

#[derive(Debug, Default)]
pub struct Informations {
    pub artists: Vec<artists::Artist>,
    pub labels: Vec<labels::Label>,
    pub musicians: Vec<musicians::Musician>,
    pub releases: Vec<releases::Release>,
}

pub fn extract() -> Informations {
    let response = reqwest::blocking::get(env::var("URL").unwrap())
        .unwrap()
        .text()
        .unwrap();

    let document = scraper::Html::parse_document(&response);

    let div_selector = scraper::Selector::parse(RECENTLY_UPDATED_SELECTOR).unwrap();

    let divs = document.select(&div_selector);

    let mut informations = Informations::default();

    for div in divs {
        if let Some(category) = div.select(&scraper::Selector::parse(CATEGORY_TITLE_SELECTOR).unwrap()).next() {

            let category_title = category.inner_html();

            let content_div = div.select(&scraper::Selector::parse(NEWS_TITLE_SELECTOR).unwrap()).next().unwrap();
            match category_title.as_str() {
                "artists" => {
                  informations.artists = artists::Artist::extract_all(scraper::Html::parse_fragment(&content_div.inner_html()));
                },
                "labels" => {
                  informations.labels = labels::Label::extract_all(scraper::Html::parse_fragment(&content_div.inner_html()));
                }
                "musicians" => {
                  informations.musicians = musicians::Musician::extract_all(scraper::Html::parse_fragment(&content_div.inner_html()));
                }
                "releases" => {
                  informations.releases = releases::Release::extract_all(scraper::Html::parse_fragment(&content_div.inner_html()));
                }
                _ => {}
            }
        }
    }

    informations
}

pub(self) fn trim_whitespace(s: &str) -> String {
    s.replace('\t', " ").replace("\n", " ").trim().to_owned()
}

pub(self) fn parse_date(s: &str) -> String {
    trim_whitespace(s)[0..19].to_owned()
}

#[cfg(test)]
mod tests {
    use super::*;
    
    #[test]
    fn test_trim_whitespace() {
        assert_eq!(trim_whitespace("Hello World"), "Hello World");
    }

    #[test]
    fn test_remove_whitespace_with_tabs() {
        assert_eq!(trim_whitespace("\tHello\tWorld\t"), "Hello World");
    }

    #[test]
    fn test_remove_whitespace_with_newlines() {
        assert_eq!(trim_whitespace("\nHello\nWorld\n"), "Hello World");
    }

    #[test]
    fn test_parse_date() {
        assert_eq!(parse_date("2022-06-16 14:12:03 by <adata-icon=\"\"data-is-vip=\"1\"href=\"/users/suji/\"class=\"usera--inherit\">suji</a>"), "2022-06-16 14:12:03");
    }
}