use scraper::Html;

use super::{Extractor, trim_whitespace, parse_date};

#[derive(Debug, Default)]
pub struct Release {
    pub artist_name: String,
    pub artist_url: String,
    pub album: String,
    pub album_url: String,
    pub date: String, 
    pub author: String,
    pub author_url: String,
}

impl Extractor for Release {
    type Output = Self;
    fn extract_all(document: Html) -> Vec<Self> {
        let mut artists : Vec<Self> = Vec::new();

        let selector = scraper::Selector::parse("ul>li").unwrap();
        let select = document.select(&selector);
        for el in select {
            artists.push(Self::extract(Html::parse_fragment(&el.html())));
        }

        artists
    }

    fn extract(document: Html) -> Self {
      let a_el = document.select(&scraper::Selector::parse("div.any--weaken-color>a.artist").unwrap()).next().unwrap();
      let title = a_el.inner_html();
      let url = a_el.value().attr("href").unwrap();

      let a_album_el = document.select(&scraper::Selector::parse("a.symbol__release").unwrap()).next().unwrap();
      let album = a_album_el.inner_html();
      let album_url = a_album_el.value().attr("href").unwrap();

      let more_info_el = document.select(&scraper::Selector::parse("div.h5").unwrap()).next().unwrap();
      let date = more_info_el.inner_html();

      let author_el = more_info_el.select(&scraper::Selector::parse("a").unwrap()).next().unwrap();
      let author = author_el.inner_html();
      let author_url = author_el.value().attr("href").unwrap();

      Self {
          artist_name: trim_whitespace(&title),
          artist_url: trim_whitespace(url),
          album: trim_whitespace(&album),
          album_url: trim_whitespace(album_url),
          date: parse_date(&date),
          author: trim_whitespace(&author),
          author_url: trim_whitespace(author_url),
      }
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    const EXAMPLES: &'static str = 
r#"
<ul>
  <li>
    <div class="any--weaken-color">
      <a class="artist" href="/artists/mucc/">MUCC</a>
    </div>
    <a class="symbol__release" href="/releases/mucc/54429/shin-sekai-tsuujouban/">Shin Sekai  Tsuujouban</a>
    <div class="h5 any--no-wrap">
      2022-06-16 12:21:00 by 
      <a class="user a--inherit" data-icon="" data-is-vip="1" href="/users/kumika/">kumika</a>
    </div>
  </li>
  <li>
    <div class="any--weaken-color">
      <a class="artist" href="/artists/lay-about-world/">LAY ABOUT WORLD</a>
      </div>
        <a class="symbol__release" href="/releases/lay-about-world/37128/c-lone/">c×lone  </a>
      <div class="h5 any--no-wrap">
      2022-06-16 11:39:52 by 
      <a class="user a--inherit" data-icon="" data-is-vip="1" href="/users/kumika/">kumika</a>
    </div>
  </li>
</ul>
"#;

  #[test]
  fn extract_all_must_return_two_result() {
    let document = scraper::Html::parse_fragment(EXAMPLES);
    let artists = Release::extract_all(document);
    assert_eq!(artists.len(), 2);
    assert_eq!(artists[0].artist_name, "MUCC");
    assert_eq!(artists[0].artist_url, "/artists/mucc/");
    assert_eq!(artists[0].album, "Shin Sekai  Tsuujouban");
    assert_eq!(artists[0].album_url, "/releases/mucc/54429/shin-sekai-tsuujouban/");
    assert_eq!(artists[0].date, "2022-06-16 12:21:00");
    assert_eq!(artists[0].author, "kumika");
    assert_eq!(artists[0].author_url, "/users/kumika/");
    assert_eq!(artists[1].artist_name, "LAY ABOUT WORLD");
    assert_eq!(artists[1].artist_url, "/artists/lay-about-world/");
    assert_eq!(artists[1].album, "c×lone");
    assert_eq!(artists[1].album_url, "/releases/lay-about-world/37128/c-lone/");
    assert_eq!(artists[1].date, "2022-06-16 11:39:52");
    assert_eq!(artists[1].author, "kumika");
    assert_eq!(artists[1].author_url, "/users/kumika/");
  }

  const EXAMPLE: &'static str = r#"
  <div class="any--weaken-color">
      <a class="artist" href="/artists/mucc/">MUCC</a>
    </div>
    <a class="symbol__release" href="/releases/mucc/54429/shin-sekai-tsuujouban/">Shin Sekai  Tsuujouban</a>
    <div class="h5 any--no-wrap">
      2022-06-16 12:21:00 by 
      <a class="user a--inherit" data-icon="" data-is-vip="1" href="/users/kumika/">kumika</a>
    </div>
  "#;

  #[test]
  fn extract_must_be_return_correct_value() {
    let document = scraper::Html::parse_fragment(EXAMPLE);
    let artist = Release::extract(document);
    assert_eq!(artist.artist_name, "MUCC");
    assert_eq!(artist.artist_url, "/artists/mucc/");
    assert_eq!(artist.album, "Shin Sekai  Tsuujouban");
    assert_eq!(artist.album_url, "/releases/mucc/54429/shin-sekai-tsuujouban/");
    assert_eq!(artist.date, "2022-06-16 12:21:00");
    assert_eq!(artist.author, "kumika");
    assert_eq!(artist.author_url, "/users/kumika/");
  }
}